from typing import List, Union

import pytest

from gcs_migration.typing.gcs_documents import IdentityMappingDict
from gcs_migration.models import StorageGateway

from gcs_migration.check.storage_gateway import (
    StorageGatewayCheck,
    SG01,
    SG02,
    SG03,
    SG04,
    SG05,
    SG06,
    SG07,
)


def test_check_sg01(storage_gateway: StorageGateway) -> None:
    sg01 = SG01()

    assert not sg01.is_ok(None)
    assert sg01.is_ok(storage_gateway)

    checker = StorageGatewayCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, SG01), res)) != []

    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG01), res)) == []


def test_check_sg02(storage_gateway: StorageGateway) -> None:
    sg02 = SG02()

    storage_gateway.display_name = None

    assert sg02.is_ok(None)
    assert not sg02.is_ok(storage_gateway)

    checker = StorageGatewayCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, SG02), res)) == []

    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG02), res)) != []

    storage_gateway.display_name = "OK"
    assert sg02.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG02), res)) == []

    storage_gateway.display_name = "\n"
    assert sg02.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG02), res)) == []


def test_check_sg03(storage_gateway: StorageGateway) -> None:
    sg03 = SG03()
    checker = StorageGatewayCheck()

    storage_gateway.display_name = "OK"

    assert sg03.is_ok(None)
    assert sg03.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG03), res)) == []

    storage_gateway.display_name = "\n"
    assert not sg03.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG03), res)) != []


def test_check_sg04(storage_gateway: StorageGateway) -> None:
    sg04 = SG04()
    checker = StorageGatewayCheck()

    storage_gateway.allowed_domains = ["domain"]

    assert sg04.is_ok(None)
    assert sg04.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG04), res)) == []

    storage_gateway.allowed_domains = None
    assert not sg04.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG04), res)) != []


def test_check_sg05(storage_gateway: StorageGateway) -> None:
    sg05 = SG05()
    checker = StorageGatewayCheck()

    storage_gateway.allowed_domains = ["domain"]

    assert sg05.is_ok(None)
    assert sg05.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG05), res)) == []

    storage_gateway.allowed_domains = []
    assert not sg05.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG05), res)) != []


@pytest.mark.parametrize(
    "domains,use_mapping,expect",
    [
        (None, False, True),
        (None, True, True),
        (["example.edu"], False, True),
        (["example.edu"], True, True),
        (["example.edu", "example.org"], False, False),
        (["example.edu", "example.org"], True, True),
    ],
)
def test_check_sg06(
    storage_gateway: StorageGateway,
    storage_gateway_identity_mapping: List[IdentityMappingDict],
    domains: Union[None, List[str]],
    use_mapping: bool,
    expect: bool,
) -> None:
    sg06 = SG06()
    checker = StorageGatewayCheck()

    storage_gateway.allowed_domains = domains
    if use_mapping:
        storage_gateway.identity_mappings = storage_gateway_identity_mapping
    else:
        storage_gateway.identity_mappings = None

    assert sg06.is_ok(None)

    assert sg06.is_ok(storage_gateway) == expect
    res = checker.check(storage_gateway)

    assert (list(filter(lambda x: isinstance(x, SG06), res)) == []) == expect


def test_check_sg07(storage_gateway: StorageGateway) -> None:
    sg07 = SG07()
    checker = StorageGatewayCheck()

    storage_gateway.load_dsi_module = None

    assert sg07.is_ok(None)
    assert sg07.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG07), res)) == []

    storage_gateway.load_dsi_module = "file"
    assert sg07.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG07), res)) == []

    storage_gateway.load_dsi_module = "google_drive"
    assert not sg07.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG07), res)) != []
