from typing import Callable
from unittest import mock

import pytest
from click.testing import CliRunner

from gcs_migration.commands.sharing_state_load import sharing_state_load
from gcs_migration.models import (
    Credential,
    GuestCollection,
    MigrationPlan,
)


@pytest.fixture
def migration_plan_existing_extra_collection(
    migration_plan: MigrationPlan,
    share_state_file_creator: Callable,
    sharing_state_dir: str,
) -> MigrationPlan:
    credential = Credential()
    credential.username = "new_user"
    credential.subject = "/C=US/O=Globus/CN=New User Certificate"
    credential.gsi_authz_callout_index = 0
    credential.identity_id = "8307e9fd-1735-4552-a4ff-ba2ef886f176"
    credential.identity_username = "new_user@globus.org"

    new_guest_collection = GuestCollection()
    new_guest_collection.id = "3ed10ee9-7aca-4e85-8b8d-02e20f996570"
    new_guest_collection.host_path = "/host-path"
    new_guest_collection.default_directory = "/host-path"
    new_guest_collection.force_encryption = True
    new_guest_collection.disable_verify = False
    new_guest_collection.force_encryption = True
    new_guest_collection.owner_id = "829320ee-e339-468d-b285-ec91e8789a5f"
    new_guest_collection.owner_string = "new_user@globus.org"
    new_guest_collection.credential = credential
    new_guest_collection.sharing_uuid = "7d91c532-99b4-4946-8c91-f03248bb5a1e"
    new_guest_collection.enable_https = False
    new_guest_collection.share_state_file = share_state_file_creator(
        new_guest_collection.sharing_uuid,
        "new_user",
        sharing_state_dir,
        new_guest_collection.host_path,
        True,
        True,
    )

    migration_plan.add_guest_collection(new_guest_collection)
    return migration_plan


def test_load_share_state_file_single(
    migration_plan_existing_extra_collection: MigrationPlan,
) -> None:
    with mock.patch.object(MigrationPlan, "load") as mock_migration_load:
        mock_migration_load.return_value = migration_plan_existing_extra_collection

        assert migration_plan_existing_extra_collection.guest_collections
        for gc in migration_plan_existing_extra_collection.guest_collections:
            gc.base_path = None
        collection_id = migration_plan_existing_extra_collection.guest_collections[0].id
        assert collection_id is not None

        runner = CliRunner()
        result = runner.invoke(sharing_state_load, [collection_id])

        assert result.exit_code == 0
        for gc in migration_plan_existing_extra_collection.guest_collections:
            if gc.id == collection_id:
                assert gc.base_path is not None
            else:
                assert gc.base_path is None


def test_load_share_state_file_single_cant_load(
    migration_plan_existing_extra_collection: MigrationPlan,
) -> None:
    with mock.patch.object(
        MigrationPlan, "load"
    ) as mock_migration_load, mock.patch.object(
        GuestCollection, "load_share_state_file"
    ):
        mock_migration_load.return_value = migration_plan_existing_extra_collection

        assert migration_plan_existing_extra_collection.guest_collections
        for gc in migration_plan_existing_extra_collection.guest_collections:
            gc.base_path = None
        collection_id = migration_plan_existing_extra_collection.guest_collections[0].id
        assert collection_id is not None

        runner = CliRunner()
        result = runner.invoke(sharing_state_load, [collection_id])

        assert result.exit_code == 1
        for gc in migration_plan_existing_extra_collection.guest_collections:
            assert gc.base_path is None


def test_load_share_state_file_bad_id(
    migration_plan_existing_extra_collection: MigrationPlan,
) -> None:
    with mock.patch.object(MigrationPlan, "load") as mock_migration_load:
        mock_migration_load.return_value = migration_plan_existing_extra_collection

        assert migration_plan_existing_extra_collection.guest_collections
        for gc in migration_plan_existing_extra_collection.guest_collections:
            gc.base_path = None
        collection_id = "a78720f9-f5f6-4167-bb69-4fa88258903b"

        runner = CliRunner()
        result = runner.invoke(sharing_state_load, [collection_id])

        assert result.exit_code == 1
        for gc in migration_plan_existing_extra_collection.guest_collections:
            assert gc.base_path is None


def test_load_share_state_file_all(
    migration_plan_existing_extra_collection: MigrationPlan,
) -> None:
    with mock.patch.object(MigrationPlan, "load") as mock_migration_load:
        mock_migration_load.return_value = migration_plan_existing_extra_collection

        assert migration_plan_existing_extra_collection.guest_collections
        for gc in migration_plan_existing_extra_collection.guest_collections:
            gc.base_path = None
        runner = CliRunner()
        result = runner.invoke(sharing_state_load, catch_exceptions=False)
        assert result.exit_code == 0
        for gc in migration_plan_existing_extra_collection.guest_collections:
            assert gc.base_path is not None


def test_load_share_state_file_all_cant_load(
    migration_plan_existing_extra_collection: MigrationPlan,
) -> None:
    with mock.patch.object(
        MigrationPlan, "load"
    ) as mock_migration_load, mock.patch.object(
        GuestCollection, "load_share_state_file"
    ):
        mock_migration_load.return_value = migration_plan_existing_extra_collection

        assert migration_plan_existing_extra_collection.guest_collections
        for gc in migration_plan_existing_extra_collection.guest_collections:
            gc.base_path = None
        runner = CliRunner()
        result = runner.invoke(sharing_state_load, catch_exceptions=False)
        assert result.exit_code == 1
        for gc in migration_plan_existing_extra_collection.guest_collections:
            assert gc.base_path is None
            assert str(gc.id) in result.output
